/**
 *
 * Unity/Compiz Dialog Handler Plugin
 *
 * unitydialog.h
 *
 * Copyright : (C) 2010 Canonical Ltd.
 *
 * Authors: Sam Spilsbury   <sam.spilsbury@canonical.com>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 **/

#include <core/core.h>
#include <composite/composite.h>
#include <opengl/opengl.h>

#include <cairo-xlib-xrender.h>

#include "unitydialog_options.h"

class UnityDialogScreen :
    public PluginClassHandler <UnityDialogScreen, CompScreen>,
    public ScreenInterface,
    public UnitydialogOptions
{
    public:

	UnityDialogScreen (CompScreen *);
	~UnityDialogScreen ();

	void
	cairoClear (cairo_t    *cr);

	cairo_t *
	cairoContext ();

	void
	renderDim ();

	void
	handleEvent (XEvent *);

	void
	optionChanged (CompOption *opt,
		       Options     num);

    public:

	CompositeScreen *cScreen;
	GLScreen	*gScreen;

    public:

	Pixmap pixmap;
	GLTexture::List texture;
	cairo_surface_t *surface;
	cairo_t		*cairo;
	Damage		damage;
	Atom		compizResizeAtom;

    private:
};

#define UNITY_DIALOG_SCREEN(s)						       \
    UnityDialogScreen *uds = UnityDialogScreen::get (s);

class UnityDialogWindow :
    public PluginClassHandler <UnityDialogWindow, CompWindow>,
    public WindowInterface,
    public GLWindowInterface
{
    public:

	UnityDialogWindow (CompWindow *w);
	~UnityDialogWindow ();

    public:

	CompWindow *window;
	CompositeWindow *cWindow;
	GLWindow   *gWindow;

    public:

	bool
	glDraw (const GLMatrix &, GLFragment::Attrib &,
		const CompRegion &, unsigned int);

	void windowNotify (CompWindowNotify n);
	void grabNotify (int x, int y,
			 unsigned int state,
			 unsigned int mask);
	void ungrabNotify ();
	void moveNotify (int x, int y, bool immediate);

	void addTransient (CompWindow *);
	void removeTransient (CompWindow *);

	void queueGrab (int, int, unsigned int, unsigned int);
	void queueUngrab ();
	void queueMove (int, int, bool);

	bool	       mSkipNotify;
	CompWindowList mTransients;
	CompWindow     *mParent;

    private:
};

#define VIG_WINDOW(w)						       \
    UnityDialogWindow *vw = UnityDialogWindow::get (w);

class UnityDialogPluginVTable :
    public CompPlugin::VTableForScreenAndWindow <UnityDialogScreen, UnityDialogWindow>
{
    public:

	bool init ();
};
